/*
 * wysmod - wysmod.c
 *	version 0.0.0
 *
 * Copyright (C) 2005..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 * Copyright (C) 2006, 2007  Marek Piwonski <marek@microster.pl>
 * Copyright (C) 2006  Krzysztof Kasperski <casper@microster.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/modbus.h>
#include <tmos/console.h>
#include <tmos/jiffies.h>
#include <tmos/watchdog.h>
#include <tmos/mb-slave.h>
#include <tmos/net.h>

#include "display.h"

/*
 * The keys are probed in quite low frequency, to catch short presses
 * we're reading keys with higher frequency and return 1 if any
 * of reads returned 1.
 */
static uint8_t keys;

/*
 * If Master did not read keys since a second it's dead.
 *
 * This is a timestamp of last read keys command.
 */
static jiffies_t last_read_time;

mb_ex_t mb_slave_hr_read(uint16_t __kernel *v, mb_addr_t addr, mb_size_t len)
	__reentrant
{
	if ((addr > 0) || (len > 1))
		return MBEX_FAULT;

	*v = cpu_to_be16((uint16_t) keys);
	last_read_time = get_jiffies();
	keys = 0;
	return 0;
}

mb_ex_t mb_slave_hr_write(const uint16_t __kernel *v, mb_addr_t addr,
		mb_size_t len) __reentrant
{
	unsigned char count = CONSOLE_COLS * CONSOLE_ROWS / 2;

	if ((addr >= count) || (addr + len > count))
		return MBEX_FAULT;

	while (len > 0) {
		uint16_t x = be16_to_cpu(*v);

		c_putchar_p(2 * addr + 0, x >> 8);
		c_putchar_p(2 * addr + 1, x & 0xff);
		v++;
		addr++;
		len--;
	}
	return 0;
}

void wysmod(void) REENTRANT
{
	jiffies_t t;

	net_ioctl(NET_IFACE_SERIAL0, NET_IOCTL_UPDOWN, NET_IOCTL_UPDOWN_UP);

	keys = 0;
	last_read_time = get_jiffies();

	for (;;) {
		watchdog_refresh();

		/* read keys */
		keys |= KEYS_PORT;

		mb_slave_recv();

		t = get_jiffies();

		if (time_after(t, last_read_time + S_JIFFIES(1)))
			break;

		/*
		 * latency is not important in this application,
		 * it's good idea to save some energy
		 */
		halt();
	}

	net_ioctl(NET_IFACE_SERIAL0, NET_IOCTL_UPDOWN, NET_IOCTL_UPDOWN_DOWN);
}
