/*
 * aduc - beersw.c
 * 	version 0.0.0
 *
 * 	beer16b SPI switches
 * 
 * Copyright (C) 2010, 2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/spi.h>
#include <tmos/sched.h>
#include <asm/aduc841.h>
#include <tmos/port.h>
#include <asm/irqs.h>

#ifdef CONFIG_BEER16B
#define enable_sw()	do { P2_0 = 1; } while (0)
#define disable_sw()	do { P2_0 = 0; } while (0)
#else
#define enable_sw()	do { } while (0)
#define disable_sw()	do { } while (0)
#endif

msg_ssize_t beersw_read(struct iocb __kernel *iocb)
	__reentrant
{
	unsigned char x;

	if (!iocb->len)
		return 0;

	enable_sw();
	x = ~spi_xmit(0);
	disable_sw();

	/* convert from BCD to number */
	x = 10 * (x >> 4) + (x & 0xf);
	iocb->buf[0] = x;
	return 1;
}

static void beersw_init(void)
{
	disable_sw();
}

device_initcall(beersw_init)
