/*
 * tmos - kernel.h
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __KERNEL_H
#define __KERNEL_H

#include <tmos/config.h>

#include <tmos/compiler.h>
#include <tmos/types.h>
#include <asm/system.h>

#ifdef CONFIG_PRINTK
int printk(const char *fmt, ...) REENTRANT;
#else
#define printk(fmt...)	do { } while (0)
#endif

#ifdef CONFIG_DEBUG
void __dump_stack(unsigned long sp) REENTRANT;
void dump_stack(void);
#else
#define __dump_stack(sp) do { } while (0)
#define dump_stack() do { } while (0)
#endif

#ifdef CONFIG_DEBUG
#define BUG() \
	do { \
		printk("BUG at %s:%d\n", __FILE__, __LINE__); \
		dump_stack(); \
		for (;;) \
			halt(); \
	} while (0)
#else
#define BUG() \
	do { \
	} while (0)
#endif

#define BUG_ON(cond) \
	do { \
		if (unlikely(cond)) \
			BUG(); \
	} while (0)

#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

#define min(a, b)	(((a) < (b)) ? (a) : (b))
#define max(a, b)	(((a) > (b)) ? (a) : (b))

#define clamp(_val, _min, _max)	\
		max(min(_val, _max), _min)

#define DIV_ROUND_CLOSEST(x, divisor) (((x) + ((divisor) / 2)) / (divisor))

/*
 * app_main() it's the main() function of application. It's called by
 * after system initialization with enabled interrupts and multitasking,
 * if enabled.
 */
int app_main(void) __reentrant;

int start_kernel(void);

#endif
